/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import java.sql.Connection;

import org.databene.commons.StringUtil;
import org.databene.commons.iterator.TableRowIterator;
import org.databene.dbsanity.model.query.DefectQueryStrategy;
import org.databene.jdbacl.model.DBTable;
import org.databene.jdbacl.model.Database;

/**
 * Verifies that an expression provides only non-null values.<br/><br/>
 * Created: 05.11.2011 07:38:05
 * @since 0.9.1
 * @author Volker Bergmann
 */
public class NotNullQueryStrategy extends DefectQueryStrategy {
	
	String tableName;
	String expression;
	String condition;
	String reportColumns;

	public NotNullQueryStrategy(String tableName, String expression, String condition, String reportColumns) {
		super(null);
		this.tableName = tableName;
		this.expression = expression;
		this.condition = condition;
		this.reportColumns = reportColumns;
	}

	@Override
	public TableRowIterator perform(Connection connection, Database database) {
		DBTable table = database.getTable(tableName);
		String reportColumns = renderReportColumns(this.reportColumns, table);
		this.query = "SELECT " + reportColumns + " FROM " + tableName + " WHERE " + expression + " IS NULL";
		if (!StringUtil.isEmpty(condition))
			this.query += " AND " + condition;
		return super.perform(connection, database);
	}
	
}
