/*
 * (c) Copyright 2010-2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model.query;

import java.sql.Connection;

import org.databene.commons.iterator.TableRowIterator;
import org.databene.commons.iterator.TableRowIteratorProxy;
import org.databene.dbsanity.model.AbstractCheckStrategy;
import org.databene.jdbacl.ArrayResultSetIterator;
import org.databene.jdbacl.model.Database;

/**
 * Performs a check based on a SQL query that lists defective database rows.<br/><br/>
 * Created: 02.12.2010 16:54:35
 * @since 0.4
 * @author Volker Bergmann
 */
public class DefectQueryStrategy extends AbstractCheckStrategy {

	protected String query;
	protected long defectCount;

	public DefectQueryStrategy(String query) {
	    this.query = query;
	    this.defectCount = 0;
	}
	
	public String getQuery() {
		return query;
	}
	
	public String getDetails() {
		return query;
	}
	
	public TableRowIterator perform(Connection connection, Database database) {
		return new Proxy(new ArrayResultSetIterator(connection, query));
	}

	public long getDefectCount() {
		return defectCount;
	}

	public void reset() {
		defectCount = 0;
	}

	protected class Proxy extends TableRowIteratorProxy {

		public Proxy(TableRowIterator source) {
	        super(source);
        }
		
		@Override
		public Object[] next() {
			// first query the source, then update the counter
			Object[] result = super.next();
			// thus, if there is an exception in the source's next() method, the counter is not increased
			defectCount++;
			return result;
		}

	}

}
